"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.kill = exports.send = exports.start = void 0;
const execa_1 = __importDefault(require("execa"));
const debug_1 = __importDefault(require("debug"));
const path_1 = require("path");
const utils_1 = require("./subprocess/utils");
const logger = debug_1.default("testing-library-spy");
let subprocess;
/**
 * Subprocess won't be able to find out whether it's running inside TTY, so we need to manually pass down the setting.
 */
function useDebugColors() {
    // Leverage undocumented debug function
    const useColors = debug_1.default.useColors;
    return useColors && useColors() ? "yes" : undefined;
}
function start() {
    if (subprocess) {
        return;
    }
    const SCRIPT_PATH = path_1.join(__dirname, "subprocess/index.js");
    const port = utils_1.getPort();
    logger("Starting subprocess...");
    subprocess = execa_1.default.node(SCRIPT_PATH, {
        env: {
            TESTING_LIBRARY_SPY_PORT: `${port}`,
            DEBUG_COLORS: useDebugColors(),
        },
    });
    logger("Subprocess started.");
    subprocess.catch((error) => {
        console.error("Testing Library Spy subprocess exited prematurely!");
        throw error;
    });
    subprocess.stdout.pipe(process.stdout);
    subprocess.stderr.pipe(process.stderr);
    console.info(`Testing Library Spy is at http://localhost:${port}/`);
}
exports.start = start;
function send(data) {
    if (!subprocess) {
        throw new Error("Testing Library Spy subprocess is not ready yet!");
    }
    logger("Sending message %o", data);
    subprocess.send(data);
}
exports.send = send;
function kill() {
    if (!subprocess) {
        return;
    }
    logger("Killing subprocess...");
    subprocess.cancel();
    subprocess = undefined;
    logger("Subprocess killed.");
}
exports.kill = kill;
//# sourceMappingURL=server.js.map