"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initPlayground = void 0;
const server_1 = require("./server");
const react_1 = require("@testing-library/react");
const inspector_1 = require("inspector");
let isInitialized = false;
let observer;
let oldValue = "";
// TODO: Report more information, e.g. test name...
// TODO: Can we report last executed query? .) E.g. if I reexport the library, then I could also provide render() method, and maybe some other stuff...
// TODO: This needs to play well with DEBUG / non-DEBUG mode.
const updateContents = () => {
    if (!isInitialized) {
        return;
    }
    const html = react_1.prettyDOM(document.body, undefined, {
        highlight: false,
    }) || "";
    if (oldValue === html) {
        return;
    }
    oldValue = html;
    server_1.send(html);
};
const hookInTestingLibrary = () => {
    const { asyncWrapper, eventWrapper } = react_1.getConfig();
    react_1.configure({
        asyncWrapper: async (...args) => {
            try {
                return await asyncWrapper(...args);
            }
            finally {
                updateContents();
            }
        },
        eventWrapper: async (...args) => {
            try {
                // const element = args[0];
                return await eventWrapper(...args);
            }
            finally {
                updateContents();
            }
        },
    });
    const oldDebug = react_1.screen.debug;
    react_1.screen.debug = (...args) => {
        updateContents();
        oldDebug(...args);
    };
};
const hookToDom = () => {
    const window = document.defaultView;
    if (!window) {
        throw new Error("Window should be initialized by JSDOM!");
    }
    const MutationObserverConstructor = window.MutationObserver;
    observer = new MutationObserverConstructor(updateContents);
    observer.observe(document.body, {
        attributes: true,
        characterData: true,
        childList: true,
        subtree: true,
    });
};
const enable = () => {
    // Enable by ENV variable?
    if (process.env.LIVE_PLAYGROUND_ENABLE === "true") {
        return true;
    }
    // Or when Node.js debugger is active?
    if (inspector_1.url()) {
        return true;
    }
    return false;
};
// TODO: Rename this to something like testing-library-spy
// TODO: Just dump out HTML nicely, with info about running test and connection status. And add link to download this package and the chrome extension
// TODO: Add here options object, with possibility to patch global object and/or screen.debug() or something to call notifyPlayground()...
exports.initPlayground = () => {
    if (!enable()) {
        return;
    }
    if (isInitialized) {
        return;
    }
    isInitialized = true;
    server_1.start();
    hookInTestingLibrary();
    hookToDom();
};
// https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver
// TODO: If we monkeypatch .debug(), can we then somehow stack what has been happening, to allow going back in time?
// TODO: How to detect afterAll/afterEach? https://github.com/testing-library/react-testing-library/blob/master/src/index.js
// TODO: Can we replace this with some unref(), so Jest doesn't bother?
// if ((afterAll !== undefined) {
//   afterAll(() => {
//     if (observer) {
//       observer.disconnect();
//     }
//     observer = undefined;
//
//     kill();
//   });
// }
// TODO: ^ Solve this
//# sourceMappingURL=toolkit.js.map